@@ -87,7 +87,12 @@ struct _GdlSwitcherPrivate {
     gboolean in_toggle;
 };
 
-G_DEFINE_TYPE (GdlSwitcher, gdl_switcher, GTK_TYPE_NOTEBOOK)
+struct _GdlSwitcherClassPrivate {
+    GtkCssProvider *css;
+};
+
+G_DEFINE_TYPE_WITH_CODE (GdlSwitcher, gdl_switcher, GTK_TYPE_NOTEBOOK,
+                         g_type_add_class_private (g_define_type_id, sizeof (GdlSwitcherClassPrivate)))
 
 #define INTERNAL_MODE(switcher)  (switcher->priv->switcher_style == \
             GDL_SWITCHER_STYLE_TOOLBAR ? switcher->priv->toolbar_style : \
@@ -663,9 +668,8 @@ gdl_switcher_dispose (GObject *object)
     }
     g_object_unref (gconf_client);
 #endif
-    
-    g_slist_foreach (priv->buttons, (GFunc) button_free, NULL);
-    g_slist_free (priv->buttons);
+
+    g_slist_free_full (priv->buttons, (GDestroyNotify)button_free);
     priv->buttons = NULL;
 
     G_OBJECT_CLASS (gdl_switcher_parent_class)->dispose (object);
@@ -674,10 +678,6 @@ gdl_switcher_dispose (GObject *object)
 static void
 gdl_switcher_finalize (GObject *object)
 {
-    GdlSwitcherPrivate *priv = GDL_SWITCHER (object)->priv;
-
-    g_free (priv);
-
     G_OBJECT_CLASS (gdl_switcher_parent_class)->finalize (object);
 }
 
@@ -750,6 +750,12 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
     GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    static const gchar button_style[] =
+       "* {\n"
+           "-GtkWidget-focus-line-width : 1;\n"
+           "-GtkWidget-focus-padding : 1;\n"
+           "padding: 0;\n"
+       "}";
 
     container_class->forall = gdl_switcher_forall;
     container_class->remove = gdl_switcher_remove;
@@ -772,15 +778,14 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
                            GDL_TYPE_SWITCHER_STYLE,
                            GDL_SWITCHER_STYLE_BOTH,
                            G_PARAM_READWRITE));
-    
-    gtk_rc_parse_string ("style \"gdl-button-style\"\n"
-                         "{\n"
-                         "GtkWidget::focus-padding = 1\n"
-                         "GtkWidget::focus-line-width = 1\n"
-                         "xthickness = 0\n"
-                         "ythickness = 0\n"
-                         "}\n"
-                         "widget \"*.gdl-button\" style \"gdl-button-style\"");
+
+    g_type_class_add_private (object_class, sizeof (GdlSwitcherPrivate));
+
+    /* set the style */
+    klass->priv = G_TYPE_CLASS_GET_PRIVATE (klass, GDL_TYPE_SWITCHER, GdlSwitcherClassPrivate);
+
+    klass->priv->css = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (klass->priv->css, button_style, -1, NULL);
 }
 
 static void
@@ -790,8 +795,10 @@ gdl_switcher_init (GdlSwitcher *switcher)
 
     gtk_widget_set_has_window (GTK_WIDGET (switcher), FALSE);
   
-    priv = g_new0 (GdlSwitcherPrivate, 1);
-    switcher->priv = priv;
+    switcher->priv = G_TYPE_INSTANCE_GET_PRIVATE (switcher,
+                                                  GDL_TYPE_SWITCHER,
+                                                  GdlSwitcherPrivate);
+    priv = switcher->priv;
 
     priv->show = TRUE;
     priv->buttons_height_request = -1;
@@ -813,8 +820,7 @@ gdl_switcher_init (GdlSwitcher *switcher)
 GtkWidget *
 gdl_switcher_new (void)
 {
-    GdlSwitcher *switcher = g_object_new (gdl_switcher_get_type (), NULL);
-    return GTK_WIDGET (switcher);
+    return g_object_new (gdl_switcher_get_type (), NULL);
 }
 
 void
@@ -830,7 +836,6 @@ gdl_switcher_add_button (GdlSwitcher *switcher, const gchar *label,
     GtkWidget *arrow;
     
     button_widget = gtk_toggle_button_new ();
-    gtk_widget_set_name (button_widget, "gdl-button");
     gtk_button_set_relief (GTK_BUTTON(button_widget), GTK_RELIEF_HALF);
     if (switcher->priv->show)
         gtk_widget_show (button_widget);
